#
# Movie selection interface rewritten as an additional plugin by moveq
# Copyright (C) 2007-2008 moveq / Suomipoeka team (suomipoeka@gmail.com)
# In case of reuse of this source code please do not remove this copyright.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	For more information on the GNU General Public License see:
#	<http://www.gnu.org/licenses/>.
#
from Components.config import *
from Components.GUIComponent import GUIComponent
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaTest
#from Tools.Directories import SCOPE_SKIN_IMAGE
from time import localtime
from enigma import eTimer, eListboxPythonMultiContent, eListbox, gFont, iServiceInformation, iRecordableService, \
	RT_HALIGN_LEFT, RT_HALIGN_RIGHT, RT_HALIGN_CENTER, eServiceReference, getBestPlayableServiceReference, eServiceCenter, loadPNG
import os

from RecordEventObserver import RecordEventObserver
from SuomipoekaTasker import spDebugOut


#funcQueue = []
# TODO: multiple timers or queue needed instead of just priority override?
gtimer = eTimer()
gtimerfunc = None
gfuncparam = None
gfuncprio = 0
def DelayedFunctionCB():
	try:
		global gtimerfunc, gfuncparam
		gtimer.stop()
		if gtimerfunc is not None:
			if gfuncparam is None:
				try:
					gtimerfunc()
				except:
					gtimerfunc(None)	# required by desc.newService(None)
			else:
				gtimerfunc(gfuncparam)
		gtimerfunc = None
		gfuncparam = None
	except Exception, e:
		spDebugOut("[spML] DelayedFunction launch exception:\n" + str(e))

global DelayedFunction
def DelayedFunction(func, param = None, delay=100, priority=0):
	global gtimer, gtimerfunc, gfuncparam, gfuncprio
	if gtimerfunc is None or priority >= gfuncprio:	# override existing if higher priority
		if gtimerfunc is not None:
			spDebugOut("[spML] DelayedFunction override: " + str((gtimerfunc, func)))
		gtimer.stop()
		gfuncparam = param
		gtimerfunc = func
		gfuncprio = priority
		gtimer.start(delay, False)
	else:
		spDebugOut("[spML] DelayedFunction not ready:\n" + str((func, param)))
gtimer.timeout.get().append(DelayedFunctionCB)



currentSelectionCount = 0
class MovieList(GUIComponent):
	def __init__(self):
		self.list = [] # to avoid blue screen when movie list opened with long VIDEO-press
		GUIComponent.__init__(self)
		self.l = eListboxPythonMultiContent()
		self.l.setFont(0, gFont("Regular", 22))
		self.l.setFont(1, gFont("Regular", 20))
		self.l.setFont(2, gFont("Regular", 16))
		self.l.setBuildFunc(self.buildMovieListEntry)
		self.l.setItemHeight(25)
		self.alphaSort = False
		self.selectionList = None
		self.recObserver = RecordEventObserver(self.recEvent)
		self.highlightsMov = []
		self.highlightsDel = []
		self.highlightsRec = []
		if os.path.exists('/usr/share/enigma2/extensions/directory.png'):
			self.dirPixmap = loadPNG('/usr/share/enigma2/extensions/directory.png')
		else:
			self.dirPixmap = loadPNG('/usr/share/enigma2/folder.png')
		self.mov1Pixmap = loadPNG('/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/movie.png')
		self.mov2Pixmap = loadPNG('/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/movie_no.png')

	def reloadWrapper(self, service):
		self.reload(self.loadPath)
#		for entry in self.list:
#			spDebugOut("[spML] compare" + str((service, entry[0])))
#			if entry[0] == service:
#				spDebugOut("[spML] ----> REC MATCH <----")

	def recEvent(self, recservice, event):
		try:
			if event == iRecordableService.evStart:
				DelayedFunction(self.reloadWrapper, recservice, 3000, 2)
				spDebugOut("[spML] record started event: " + str(recservice))
				try:
					spDebugOut("[spML] test 1: " + str( eServiceReference(recservice).getPath() ) )
				except:
					pass
				spDebugOut("[spML] test 2: " + str( eServiceReference(recservice).getServiceName() ) )
			elif event == iRecordableService.evEnd:
				pass
		except Exception, e:
			spDebugOut("[spML] recEvent exception:\n" + str(e))

	def setAlphaSort(self, trueOrFalse):
		self.alphaSort = trueOrFalse

	def buildMovieListEntry(self, serviceref, info, mlen, datesort, moviestring, filename, selnum):
		pixmapColours = {"backcolor": 0x3300071e, "backcolor_sel": 0x335b7395}
		if datesort is None:
			res = [ None ]
			if config.suomipoeka.movie_dateleft.value:
				res.append(MultiContentEntryPixmapAlphaTest(pos=(8,0), size=(24,24), png=self.dirPixmap, **pixmapColours))
				res.append(MultiContentEntryText(pos=(80, 0), size=(310, 22), font=1, flags=RT_HALIGN_LEFT, text=filename))
#				res.append(MultiContentEntryText(pos=(485, 0), size=(24, 22), font=1, flags=RT_HALIGN_LEFT, text=selnumtxt))
			else:
#				res.append(MultiContentEntryText(pos=(0, 0), size=(24, 22), font=1, flags=RT_HALIGN_LEFT, text=selnumtxt))
				res.append(MultiContentEntryPixmapAlphaTest(pos=(20,2), size=(20,20), png=self.dirPixmap, **pixmapColours))
				res.append(MultiContentEntryText(pos=(50, 0), size=(310, 22), font=1, flags=RT_HALIGN_LEFT, text=filename))
				res.append(MultiContentEntryText(pos=(366, 0), size=(200, 22), font=1, flags=RT_HALIGN_LEFT, text=_("Directory")))
			return res
		if serviceref.flags & eServiceReference.mustDescent:
			return None

		if mlen > 0:
			mlen = " %dm" % ((mlen+30) / 60)
		else:
			mlen = "" # empty string until a value is fetched

		date = "---.---  "
		time = "---:---"

		if datesort != "0000":
			# datesort = YYMMDDTTTT
			time = datesort[6:8] + ":" + datesort[8:10]
			date = datesort[4:6] + "." + datesort[2:4] + "." + "  " #+ datesort[0:2]

		selnumtxt = ""
		if selnum == 9999: selnumtxt = "[M]"
		elif selnum == 9998: selnumtxt = "[D]"
		elif selnum == 9997: selnumtxt = "[R]"
		elif selnum > 0: selnumtxt = "%02d" % selnum
		if serviceref in self.highlightsMov: selnumtxt = "[M]"
		elif serviceref in self.highlightsDel: selnumtxt = "[D]"
		elif serviceref in self.highlightsRec: selnumtxt = "[R]"

		pixmap = self.mov1Pixmap
		if not moviestring.endswith(" ") and config.suomipoeka.movie_mark.value:
			pixmap = self.mov2Pixmap

		res = [ None ]
		if config.suomipoeka.movie_dateleft.value:
			res.append(MultiContentEntryPixmapAlphaTest(pos=(0,2), size=(20,20), png=pixmap, **pixmapColours))
			res.append(MultiContentEntryText(pos=(22, 0), size=(60, 22), font=1, flags=RT_HALIGN_LEFT, text=date))
			res.append(MultiContentEntryText(pos=(100, 0), size=(400, 22), font=1, flags=RT_HALIGN_LEFT, text=moviestring))
			res.append(MultiContentEntryText(pos=(495, 0), size=(24, 22), font=1, flags=RT_HALIGN_CENTER, text=selnumtxt))
		else:
			res.append(MultiContentEntryText(pos=(0, 0), size=(26, 22), font=1, flags=RT_HALIGN_LEFT, text=selnumtxt))
			res.append(MultiContentEntryPixmapAlphaTest(pos=(26,2), size=(20,20), png=pixmap, **pixmapColours))
			res.append(MultiContentEntryText(pos=(48, 0), size=(290, 22), font=1, flags=RT_HALIGN_LEFT, text=moviestring))
			res.append(MultiContentEntryText(pos=(385, 0), size=(85, 22), font=1, flags=RT_HALIGN_RIGHT, text=date))
			res.append(MultiContentEntryText(pos=(340, 0), size=(65, 22), font=1, flags=RT_HALIGN_LEFT, text=mlen))
			res.append(MultiContentEntryText(pos=(470, 0), size=(85, 22), font=1, flags=RT_HALIGN_LEFT, text=time))
		return res

	def moveToIndex(self, index):
		self.instance.moveSelectionTo(index)

	def getCurrentIndex(self):
		return self.instance.getCurrentIndex()

	def getCurrentEvent(self):
		l = self.l.getCurrentSelection()
		return l and l[0] and l[1] and l[1].getEvent(l[0])

	def getCurrent(self):
		l = self.l.getCurrentSelection()
		return l and l[0]

	GUI_WIDGET = eListbox
	
	def postWidgetCreate(self, instance):
		instance.setWrapAround(True)
		instance.setContent(self.l)

	def removeService(self, service):
		for l in self.list[:]:
			if l[0] == service:
				self.list.remove(l)
		self.l.setList(self.list)

	def serviceBusy(self, service):
		return service in self.highlightsMov or service in self.highlightsDel

	def serviceMoving(self, service):
		return service in self.highlightsMov

	def serviceDeleting(self, service):
		return service in self.highlightsDel

	def serviceRecording(self, service):
		return service in self.highlightsRec

	def highlightService(self, enable, mode, service):
		if enable:
			if mode == "move":
				self.highlightsMov.append(service)
				self.toggleSelection(service, 9999)
			elif mode == "del":
				self.highlightsDel.append(service)
				self.toggleSelection(service, 9998)
			elif mode == "rec":
				self.highlightsRec.append(service)
				self.toggleSelection(service, 9997)
		else:
			if mode == "move":
				self.highlightsMov.remove(service)
			elif mode == "del":
				self.highlightsDel.remove(service)
			elif mode == "rec":
				self.highlightsRec.remove(service)

	def __len__(self):
		return len(self.list)

	def makeSelectionList(self):
		selList = [ ]
		global currentSelectionCount
		if currentSelectionCount == 0:
			# if no selections made, select the current cursor position
			single = self.l.getCurrentSelection()
			if single:
				if single[2] is None:	# directory
					selList.append([single[0], single[5]])
				else:
					selList.append([single[0], None])
		else:
			selList = self.selectionList
#		spDebugOut("[spML] makeSelectionList: " + str(selList))
		return selList

	def resetSelectionList(self):
		self.selectionList = None
		global currentSelectionCount
		currentSelectionCount = 0

	def toggleSelection(self, service=None, overrideNum=None):
		global currentSelectionCount
		x = None
		index = 0
		if service is None:
			current = self.l.getCurrentSelection()
			if current is None: return
			index = self.getCurrentIndex()
			x = self.list[index]
		else:
			for e in self.list:
				if e[0] == service:
					x = e
					break
				index += 1

		if self.selectionList == None:
			self.selectionList = [ ]

		newselnum = x[6]	# init with old selection number
		if overrideNum == None:
			if self.serviceBusy(x[0]): return	# no toggle if file being operated on
			# basic selection toggle
			if newselnum == 0:
				# was not selected
				currentSelectionCount += 1
				newselnum = currentSelectionCount
				if current:
					self.selectionList.append([current[0], None]) # store the movie data
			else:
				# was selected, reset selection number and decrease all that had been selected after this
				newselnum = 0
				currentSelectionCount -= 1
				count = 0
				if current:
					self.selectionList.remove([current[0], None]) # remove the movie data
				for i in self.list:
					if i[6] > x[6]:
						self.list.remove(self.list[count])
						self.list.insert(count, (i[0],i[1],i[2],i[3],i[4],i[5],i[6]-1))
						self.l.invalidateEntry(count) # force redraw
					count += 1
		else:
			newselnum = overrideNum * (newselnum == 0)

		self.list.remove(self.list[index])
		self.list.insert(index, (x[0],x[1],x[2],x[3],x[4],x[5],newselnum))
		self.l.invalidateEntry(index) # force redraw of the modified item

	def updateLengthOfIndex(self, index):
		if len(self.list) > index:
			x = self.list[index]
			# (service, info, length, sortdate, sortstring, filename, selectionnum)
			self.list[index] = (x[0], x[1], x[1].getLength(x[0]), x[3], x[4], x[5], x[6])
			self.l.invalidateEntry(index)

	def getFileNameOfService(self, service):
		for entry in self.list:
			if entry[0] == service:
				return entry[5]
		return None

	def createDirlist(self, loadPath):
		subdirlist = []
		dirlist = os.listdir(loadPath)	# only need to deal with spaces when executing in shell
		# add sub directories to the list
		for p in dirlist:
			if os.path.isdir(loadPath + p):
				if config.suomipoeka.movie_trashcan_hide.value:
					# TODO: dereference links for comparison?
					if (loadPath + p) == config.suomipoeka.movie_trashpath.value:
						continue
				subdirlist.append(p)
		subdirlist.sort(key=str.lower)
		if loadPath != "/" and loadPath[:-1] != config.suomipoeka.movie_pathlimit.value:
			subdirlist.insert(0, "..")
		return subdirlist

	def reload(self, loadPath):
		self.loadPath = loadPath
		subdirlist = self.createDirlist(loadPath)

		root = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + loadPath)
		self.list = []
		self.serviceHandler = eServiceCenter.getInstance()
		list = self.serviceHandler.list(root)

		global currentSelectionCount
		currentSelectionCount = 0
		self.selectionList = None

		while 1:
			serviceref = list.getNext()
			if not serviceref.valid():
				break
			if serviceref.flags & eServiceReference.mustDescent:
				continue
			info = self.serviceHandler.info(serviceref)
			if info is None:
				continue

			moviestring = ""
			mpath = serviceref.getPath()
			filename = mpath.split("/")[-1]
			date = "0000"
			if filename[0:8].isdigit() and filename[9:13].isdigit() and not filename[8:1].isdigit():
				date = filename[2:8] + filename[9:13]
# > meta load
			if os.path.exists(mpath + ".meta") and config.suomipoeka.movie_metaload.value:
				file = open(mpath + ".meta", "r")
				file.readline()
				line = file.readline()
				file.close()
				moviestring = line.rstrip("\r\n")
			else:
# < meta load
				moviestring = filename.replace(".ts","").replace("_ "," ").replace("_"," ")
				if moviestring[0:8].isdigit() and moviestring[9:13].isdigit() and not moviestring[8:1].isdigit():
					moviestring = moviestring[16:]	# skips "YYYYMMDD TIME - "
					chlMarker = moviestring.find(" - ")
					if chlMarker > 0:
						moviestring = moviestring[3+chlMarker:]

			if os.path.exists(mpath + ".cuts"):	# watched / not watched indicator
				moviestring += " "

			if not (self.serviceMoving(serviceref) and config.suomipoeka.movie_hide_mov.value):
				if not (self.serviceDeleting(serviceref) and config.suomipoeka.movie_hide_del.value):
					# TODO: cleanup table structure
					# (service, info, length, sortdate, sortstring, filename, selnum)
					self.list.append((serviceref, info, -1, date, moviestring, filename, 0))

		if self.alphaSort == True:
			self.list.sort(key=lambda x: x[4],reverse=config.suomipoeka.movielist_reversed.value)
		else:
			self.list.sort(key=lambda x: x[3],reverse= not config.suomipoeka.movielist_reversed.value)

		# add sub directories to the list
		if subdirlist is not None:
			dcount = 0
			for dirp in subdirlist:
				sref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + dirp)	# dummy ref
				info = self.serviceHandler.info(sref)
				self.list.insert(dcount, (sref, info, None, None, None, dirp, 0))
				dcount += 1
		self.l.setList(self.list)
		self.resetSelectionList()

	def currentSelIsDirectory(self):
		try:
			return (self.list[self.getCurrentIndex()][2] == None)
		except:
			return False

	def getCurrentSelDir(self):
		return self.list[self.getCurrentIndex()][5]

	def moveTo(self, serviceref):
		found = 0
		count = 0
		for x in self.list:
			if x[0] == serviceref:
				found = count
			count += 1
		self.instance.moveSelectionTo(found)

	def moveDown(self):
		self.instance.moveSelection(self.instance.moveDown)
