#
# New movie player written by moveq
# Copyright (C) 2007-2008 moveq / Suomipoeka team (suomipoeka@gmail.com)
# In case of reuse of this source code please do not remove this copyright.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	For more information on the GNU General Public License see:
#	<http://www.gnu.org/licenses/>.
#
from enigma import iPlayableService
from Components.ActionMap import HelpableActionMap
from Components.ServiceEventTracker import ServiceEventTracker
from Screens.Screen import Screen

from SuomipoekaTasker import spDebugOut

from Screens.InfoBarGenerics import InfoBarSeek
from Screens.InfoBar import MoviePlayer

class MoviePlayerSP(MoviePlayer):
	ENABLE_RESUME_SUPPORT = True
	ALLOW_SUSPEND = True

	def __init__(self, session, playlist, recordings):
		# MoviePlayer.__init__ will start playback for the given service!!
		MoviePlayer.__init__(self, session, session.nav.getCurrentlyPlayingServiceReference())
		self.skinName = "MoviePlayer"

		self["actions"] = HelpableActionMap(self, "MoviePlayerActions",
			{
				"leavePlayer": (self.leavePlayer, _("leave movie player..."))
			})

		# set tracker for the End Of File event
		self.__event_tracker = ServiceEventTracker(screen=self, eventmap=
			{
				iPlayableService.evEOF: self.evEOF,
			})

		self.playInit(playlist, recordings)
		self.onShown.append(self.onDialogShow)

	def playInit(self, playlist, recordings):
		self.lastservice = self.session.nav.getCurrentlyPlayingServiceReference()
		self.playlist = playlist
		self.playcount = -1
		self.recordings = recordings
		self.firstStart = False
		self.movieSelectionOpen = False
		self.curretlyPlaying = None
		self.recordings.setPlayerInstance(self)

	def onDialogShow(self):
		if self.firstStart:
			return
		self.firstStart = True
		self.evEOF()	# begin playback

	def evEOF(self):
		# see if there are more to play
		if (self.playcount+1) < len(self.playlist):
			self.is_closing = False
			self.playcount += 1
			service = self.playlist[self.playcount][0]
# TODO: remove ---> linkTargetPath = self.playlist[self.playcount][1]
			self.curretlyPlaying = service
			self.session.nav.playService(service)	# TODO: check that the file exists!
			self.currentlyPlaying = service
			self.setSeekState(InfoBarSeek.SEEK_STATE_PLAY)
			self.doSeek(0)
		else:
			self.curretlyPlaying = None
			if not self.movieSelectionOpen: # if True then close() is called in movieSelected()
				self.stopPlayback()
				self.closePrepare()
				self.close()
				spDebugOut("[spPlayer] Player closed by EOF")
			else:
				self.is_closing = True
				self.stopPlayback()

	def leavePlayer(self):
		self.stopPlayback()
		self.closePrepare()
		self.close()
		spDebugOut("[spPlayer] Player closed by user")

	def stopPlayback(self):
		try:
			self.session.nav.playService(self.lastservice)
		except Exception, e:
			spDebugOut("[spPlayer] TV service fallback exception:\n" + str(e))

	def closePrepare(self):
		self.recordings.setPlayerInstance(None)

	def showMovies(self):
		self.movieSelectionOpen = True
		self.session.execDialog(self.recordings)

	def currentlyPlayedMovie(self, wasDeleted=False):
		if wasDeleted:
			self.evEOF()	# force playback of the next movie or close the player if none left
			return None
		else:
			return self.curretlyPlaying

	def movieSelected(self, playlist):
		self.movieSelectionOpen = False
		if playlist is not None and len(playlist) > 0:
			self.playcount = -1
			self.playlist = playlist
			self.evEOF()	# start playback of the first movie
		else:
			if self.is_closing:
				self.closePrepare()
				self.close(False) # False = indication to InfoBar that the MovieSelection is not to be re-opened
				spDebugOut("[spPlayer] Delayed player close by EOF")

