#
# Movie selection interface rewritten as an additional plugin by moveq
# Copyright (C) 2007-2008 moveq / Suomipoeka team (suomipoeka@gmail.com)
# In case of reuse of this source code please do not remove this copyright.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	For more information on the GNU General Public License see:
#	<http://www.gnu.org/licenses/>.
#
from Screens.Screen import Screen
from Components.ActionMap import ActionMap
from Components.Button import Button
from Components.config import *
from Components.Label import Label
from Components.Pixmap import Pixmap
from Components.Sources.ServiceEvent import ServiceEvent
from Screens.MessageBox import MessageBox
from Tools import Notifications
from time import localtime

from enigma import eServiceReference, eServiceCenter, eTimer, eSize
import os

from MovieList import MovieList, DelayedFunction
from MovieSelectionMenu import MovieMenu
from MoviePlayer import MoviePlayerSP
from SuomipoekaTasker import spTasker, spDebugOut
from Suomipoeka import SuomipoekaAutoremove

LastPlayedMovie = None


def mountpoint(path, first=True):
	if first: path = os.path.realpath(path)
	if os.path.ismount(path) or len(path)==0: return path
	return mountpoint(os.path.split(path)[0], False)


class SelectionEventInfo:
	def __init__(self):
		self["DescriptionBorder"] = Pixmap()
		self["Service"] = ServiceEvent()

	def updateEventInfo(self):
		if not config.suomipoeka.movie_descdis.value:
			try:
				if self["list"].currentSelIsDirectory():
					self["Service"].newService(None)
				else:
					serviceref = self.getCurrent()
					self["Service"].newService(serviceref)
			except Exception, e:
				spDebugOut("[spML] updateEventInfo exception:\n" + str(e))

	def descFieldInit(self):
		if config.suomipoeka.movie_descdis.value:
			width = self["list"].instance.size().width()
			height = self.WINDOW_HEIGHT - self["key_red"].instance.size().height()
			self["list"].instance.resize(eSize(width, height))
			self["DescriptionBorder"].hide()
		else:
			width = self["list"].instance.size().width()
			height = self.WINDOW_HEIGHT - self["key_red"].instance.size().height() - self["DescriptionBorder"].instance.size().height() + 2
			self["list"].instance.resize(eSize(width, height))
			self["DescriptionBorder"].show()
			self.updateEventInfo()


class MovieSelectionSP(Screen, SelectionEventInfo):
	WINDOW_HEIGHT = 490

	skin = """
	<screen name="MovieSelectionSP" position="75,63" size="560,496" title=" ">
		<ePixmap name="red" position="0,0" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-red.png" transparent="1" alphatest="on" />
		<ePixmap name="green" position="140,0" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-green.png" transparent="1" alphatest="on" />
		<ePixmap name="yellow" position="280,0" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-yellow.png" transparent="1" alphatest="on" />
		<ePixmap name="blue" position="420,0" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-blue.png" transparent="1" alphatest="on" />
		<widget name="key_red" position="0,0" zPosition="5" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="#ffffff" shadowColor="#000000" shadowOffset="-1,-1" />
		<widget name="key_green" position="140,0" zPosition="5" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="#ffffff" shadowColor="#000000" shadowOffset="-1,-1" />
		<widget name="key_yellow" position="280,0" zPosition="5" size="140,40" valign="center" halign="center"  font="Regular;21" transparent="1" foregroundColor="#ffffff" shadowColor="#000000" shadowOffset="-1,-1" />
		<widget name="key_blue" position="420,0" zPosition="5" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="#ffffff" shadowColor="#000000" shadowOffset="-1,-1" />
		<widget name="list" position="5,42" size="555,350" zPosition="2" backgroundColor="#33000720" scrollbarMode="showOnDemand" />

		<widget name="DescriptionBorder" position="0,395" size="560,101" pixmap="skin_default/border_eventinfo.png" zPosition="3" transparent="1" alphatest="on" />

		<widget source="Service" render="Label" position="5,398" size="155,22" foregroundColor="#00bab329" font="Regular;18" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" >
			<convert type="MovieInfo">RecordServiceName</convert>
		</widget>

		<widget source="Service" render="Label" position="5,419" size="550,57" font="Regular;16" foregroundColor="#cccccc" zPosition="1" >
			<convert type="MovieInfo">ShortDescription</convert>
		</widget>

		<widget source="Service" render="Label" position="5,474" size="275,19" foregroundColor="#00bab329" font="Regular;17" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" >
			<convert type="ServiceTime">StartTime</convert>
			<convert type="ClockToText">Date</convert>
		</widget>

		<widget source="Service" render="Label" position="285,474" size="50,19" foregroundColor="#00bab329" font="Regular;17" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" >
			<convert type="ServiceTime">StartTime</convert>
			<convert type="ClockToText" />
		</widget>

		<widget source="Service" render="Label" position="335,474" size="100,19" foregroundColor="#00bab329" font="Regular;17" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" >
			<convert type="ServiceTime">EndTime</convert>
			<convert type="ClockToText">Format:- %H:%M </convert>
		</widget>

		<widget source="Service" render="Label" position="440,474" size="60,19" foregroundColor="#00bab329" font="Regular;17" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" >
			<convert type="ServiceTime">Duration</convert>
			<convert type="ClockToText">AsLength</convert>
		</widget>
	</screen>"""
	def __init__(self, session):
		Screen.__init__(self, session)
		SelectionEventInfo.__init__(self)
		self.skin = MovieSelectionSP.skin
		self.skinName = "MovieSelectionSP"
		self.wasClosed = True
		self.playerInstance = None

		self.currentGrnText = _("Alpha sort")
		self["list"] = MovieList()
		self["key_red"] = Button()
		self["key_green"] = Button()
		self["key_yellow"] = Button()
		self["key_blue"] = Button()
		self["actions"] = ActionMap(["OkCancelActions", "MovieSelectionActions", "ColorActions", "InfobarActions", "DirectionActions"],
			{
				"cancel": self.abort,
				"ok": self.movieSelected,
				"showEventInfo": self.showEventInformation,
				"contextMenu": self.openMenu,

				"red": self.deleteFile,
				"green": self.toggleSort,
				"yellow": self.moveMovie,
				"blue": self.blueFunc,
				"leftUp": self.keyPress,
				"rightUp": self.keyPress,
				"upUp": self.keyPress,
				"downUp": self.keyPress,
				"showMovies": self.selectionToggle,	# "video" button, "showRadio" not usable
				"showTv": self.reloadList,	# "audio" button
			})
		if 0:
			#todo
			self["actions"] = ActionMap(["OkCancelActions", "MovieSelectionActions", "ColorActions", "InfobarActions", "DirectionActions"],
				{
					"ok":				(self.movieSelected, _("Play selected movie(s)")),
					"cancel":			(self.abort, _("Close movie list")),
					"contextMenu":		(self.openMenu, _("Open menu")),
					"showEventInfo":	(self.showEventInformation, _("Show event info")),
					"red":				(self.deleteFile, _("Delete file or empty dir")),
					"green":			(self.toggleSort, _("Toggle date/alpha sort")),
					"yellow":			(self.moveMovie, _("Move selected movie(s)")),
					"blue":				(self.blueFunc, _("Movie home / Play last (configurable)")),
					"leftUp":			(self.keyPress, _("Move cursor page up")),
					"rightUp":			(self.keyPress, _("Move cursor page down")),
					"upUp":				(self.keyPress, _("Move cursor up")),
					"downUp":			(self.keyPress, _("Move cursor down")),
					"showMovies":		(self.selectionToggle, _("Toggle multiselection")),
					"showTv":			(self.reloadList, _("Reload movie file list")),
				})
		self["actions"].csel = self

		self.currentPathSel = config.suomipoeka.movie_homepath.value
		self.parentStack = []
		self.onExecBegin.append(self.onDialogShow)
		DelayedFunction(self.reloadList, False, 6000)	# try to read the movie home directory after Enigma is started

	def keyPress(self):		# these only catch the key-Up events, not key-Down or repeat
		DelayedFunction(self.updateAfterKeyPress, None, config.suomipoeka.movie_descdelay.value)

	def updateAfterKeyPress(self):
		self.updateTitle()
		self.updateEventInfo()

	def selectionToggle(self):
		if not self["list"].currentSelIsDirectory():
			self["list"].updateLengthOfIndex(self["list"].getCurrentIndex())
			self["list"].toggleSelection()

	def showEventInformation(self):
		from Screens.EventView import EventViewSimple
		from ServiceReference import ServiceReference
		evt = self["list"].getCurrentEvent()
		if evt:
			self.session.open(EventViewSimple, evt, ServiceReference(self.getCurrent()))

	def onDialogShow(self):
		if self.wasClosed:
			self.purgeExpiredFromTrash()
			self["key_red"].text = _("Delete")
			self["key_green"].text = self.currentGrnText
			self["key_yellow"].text = _("Move")
			self["key_blue"].text = _(config.suomipoeka.movie_bluefunc.value)
			self.initCursor()
			self.updateTitle()
			SuomipoekaAutoremove()
			if config.suomipoeka.movie_reload.value or self["list"].__len__() == 0:
				DelayedFunction(self.reloadList, None, 80, 1)
		self.wasClosed = False
		try:
			self.descFieldInit()
		except:
			pass

	def initCursor(self):
		if self.playerInstance is None:	# detect movie player state (None == not open)
			if config.suomipoeka.movielist_gotonewest.value:
				self.cursorToNewest()
		else:
			if config.suomipoeka.movielist_gotonewestp.value:
				self.cursorToNewest()
			else:
				service = self.playerInstance.currentlyPlayedMovie()
				if service is not None:
					self["list"].moveTo(service)

	def cursorToNewest(self):
		if config.suomipoeka.movielist_reversed.value:
			self["list"].moveToIndex(self["list"].__len__()-1)
		else:
			self["list"].moveToIndex(0)

	def blueFunc(self):
		if config.suomipoeka.movie_bluefunc.value == "Movie home": self.movieHome()
		elif config.suomipoeka.movie_bluefunc.value == "Play last": self.playLast()

	def movieHome(self):
		self.parentStack[:] = []	# clear the list
		self.currentPathSel = config.suomipoeka.movie_homepath.value
		self.reloadList()
		self.cursorToNewest()

	def setPlayerInstance(self, player):
		self.playerInstance = player
		if player is None:	# this means that the player was just closed
			if config.suomipoeka.movie_reopen.value:
				self.session.execDialog(self)

	def openPlayer(self, playlist):
		if self.playerInstance is None:
			Notifications.AddNotification(MoviePlayerSP, playlist, self)
		else:
			DelayedFunction(self.playerInstance.movieSelected, playlist, 30, 100)	# prio = 100, make sure nothing overrides playback start

	def playLast(self):
		global LastPlayedMovie
		if LastPlayedMovie is None:
			self.session.open(MessageBox, _("Last played movie/playlist not available..."), MessageBox.TYPE_ERROR, 10)
		else:
			self.wasClosed = True
			self.close(None)
			self.openPlayer(LastPlayedMovie)

	def moveTo(self):
		self.updateAfterKeyPress()

	def getCurrent(self):
		return self["list"].getCurrent()

	def delPathSel(self, path):
		if path != "..":
			path = self.currentPathSel+ "/" + path
			if os.path.exists(path):
				if len(os.listdir(path))>0:
					if path == config.suomipoeka.movie_trashpath.value:
						# offer to empty trash
						self.session.openWithCallback(self.emptyTrash, MessageBox, _("Permanently delete all files in trashcan?"), MessageBox.TYPE_YESNO)
					else:
						self.session.open(MessageBox, _("Directory is not empty."), MessageBox.TYPE_ERROR, 10)
				else:
					spTasker.shellExecute('rmdir "' + path +'"')
					return True
		else:
			self.session.open(MessageBox, _("Cannot delete the parent directory."), MessageBox.TYPE_ERROR, 10)
		return False

	def setNextPathSel(self, nextdir):
		moveIdx = 0
		if nextdir == "..":
			if self.currentPathSel != "" and self.currentPathSel != "/":
				if len(self.parentStack) > 0:
					popped = self.parentStack.pop()
					self.currentPathSel = popped[0]
					moveIdx = popped[1]
				else:
					# in this case we'll just go to the idx 0
					self.currentPathSel = os.path.split(self.currentPathSel)[0]
		else:
			if os.path.exists(self.currentPathSel+ "/" + nextdir):
				self.parentStack.append((self.currentPathSel, self["list"].getCurrentIndex()))
				if self.currentPathSel == "/": self.currentPathSel = ""
				self.currentPathSel += "/" + nextdir
		self.reloadList()
		self["list"].moveToIndex(moveIdx)

	def movieSelected(self):
		try:
			global LastPlayedMovie
			current = self.getCurrent()
			if current is not None:
				if self["list"].currentSelIsDirectory():
					nextPath = self["list"].getCurrentSelDir()
					self.setNextPathSel(nextPath)
				else:
					playlist = self["list"].makeSelectionList()
					if not self["list"].serviceBusy(playlist[0][0]):
						LastPlayedMovie = playlist
						self.wasClosed = True
						self.close()
						self.openPlayer(LastPlayedMovie)
					else:
						self.session.open(MessageBox, _("File not available."), MessageBox.TYPE_ERROR, 10)
		except Exception, e:
			spDebugOut("[spML] movieSelected exception:\n" + str(e))

	def openMenu(self):
		current = self.getCurrent()
		if self["list"].currentSelIsDirectory(): current = None
		self.session.openWithCallback(self.menuCallback, MovieMenu, current, self.currentPathSel)

	def menuCallback(self, selection=None):
		if selection is not None:
			if selection == "Play last": self.playLast()
			elif selection == "Movie home": self.movieHome()
			elif selection == "reload": self.reloadList()
			elif selection == "ptrash": self.emptyTrash(True)
			elif selection == "delete": self.deleteFile(True)

	def deleteFile(self, permanently=False):
		self.permanentDel  = permanently or config.suomipoeka.movie_trashcan_limit.value == 0 
		self.permanentDel |= self.currentPathSel == config.suomipoeka.movie_trashpath.value
		self.permanentDel |= mountpoint(self.currentPathSel) != mountpoint(config.suomipoeka.movie_trashpath.value)
		current = self.getCurrent()	# make sure there is atleast one entry in the list
		if current is not None:
			selectedlist = self["list"].makeSelectionList()
			if self["list"].currentSelIsDirectory() and len(selectedlist) == 1 and current==selectedlist[0][0]:
					# try to delete an empty directory
					if self.delPathSel(selectedlist[0][1]):
						self["list"].removeService(selectedlist[0][0])
						#self.lastPlayedCheck(selectedlist[0][0])
			else:
				if self["list"].serviceBusy(selectedlist[0][0]): return
				if selectedlist and len(selectedlist)>0:
					self.deleteMovieQ(selectedlist)

	def deleteMovieQ(self, selectedlist):
		try:
			self.moviesToDelete = selectedlist
			self.delCurrentlyPlaying = False

			if self.playerInstance is not None:
				nowPlaying = self.playerInstance.currentlyPlayedMovie()
				if nowPlaying:
					for entry in selectedlist:
						if entry[0] == nowPlaying:
							self.delCurrentlyPlaying = True

			entrycount = len(selectedlist)
			delStr = ("Permanently "*self.permanentDel + "Delete ").replace(" D"," d")
			if entrycount == 1:
				service = selectedlist[0][0]
				name = self["list"].getFileNameOfService(service).replace(".ts","")
				if not self.delCurrentlyPlaying:
					self.session.openWithCallback(self.deleteMovieConfimation, MessageBox, _(delStr) + name + "?", MessageBox.TYPE_YESNO)
				else:
					self.session.openWithCallback(self.deleteMovieConfimation, MessageBox, _(delStr + "currently played\n\n") + name + "?", MessageBox.TYPE_YESNO)
			else:
				if entrycount > 1:
					movienames = "\n\n"
					i = 0
					for entry in selectedlist:
						if i >= 5 and entrycount > 5:	# show only 5 entries in the file list
							movienames += "..."
							break
						i += 1
						service = entry[0]	# entry[1] = link path
						name = self["list"].getFileNameOfService(service).replace(".ts","")	# TODO: None check
						if len(name) > 48:
							name = name[:48] + "..."	# limit the name string
						movienames += name + "\n"*(i<entrycount)
					if not self.delCurrentlyPlaying:
						self.session.openWithCallback(self.deleteMovieConfimation, MessageBox, _(delStr+"all selected video files?")+movienames, MessageBox.TYPE_YESNO)
					else:
						self.session.openWithCallback(self.deleteMovieConfimation, MessageBox, _(delStr+"all selected video files? The currently playing movie is also one of the selections and its playback will be stopped.")+movienames, MessageBox.TYPE_YESNO)
				else:
					self.session.open(MessageBox, _("Delete error: no file given."), MessageBox.TYPE_ERROR)
		except Exception, e:
			self.session.open(MessageBox, _("Delete error:\n" + str(e)), MessageBox.TYPE_ERROR)
			spDebugOut("[spML] deleteMovieQ exception:\n" + str(e))

	def deleteMovieConfimation(self, confirmed):
		if confirmed and self.moviesToDelete is not None and len(self.moviesToDelete)>0:
			if self.delCurrentlyPlaying:
				if self.playerInstance is not None:
					self.playerInstance.currentlyPlayedMovie(True)	# True indicates that the currently played was deleted
			delete = config.suomipoeka.movie_trashcan_limit.value==0 or self.permanentDel
			if os.path.exists(config.suomipoeka.movie_trashpath.value) or delete:
				# if the user doesn't want to keep the movies in the trash, purge immediately
				self.execFileOp(config.suomipoeka.movie_trashpath.value, self.moviesToDelete, op="delete", purgeTrash=delete)
				for x in self.moviesToDelete:
					self.lastPlayedCheck(x)
				self["list"].resetSelectionList()
			elif not delete:
				self.session.openWithCallback(self.trashcanCreate, MessageBox, _("Delete failed because the trashcan directory does not exist. Attempt to create it now?"), MessageBox.TYPE_YESNO)
			self.moviesToDelete = None
			self.updateAfterKeyPress()

	def lastPlayedCheck(self, service):
		global LastPlayedMovie
		try:
			if LastPlayedMovie is not None:
				for old in LastPlayedMovie:
					if old[0] == service:
						LastPlayedMovie.remove(old)
						break
				if len(LastPlayedMovie) == 0:
					LastPlayedMovie = None
		except Exception, e:
			LastPlayedMovie.remove(old)
			spDebugOut("[spML] lastPlayedCheck exception:\n" + str(e))

	def abort(self):
		current = self.getCurrent()
		self.wasClosed = True
		self.close(None)
		if self.playerInstance is not None:
			self.playerInstance.movieSelected(None)	# the player is still open and this will close it

	def toggleSort(self):
		if self["key_green"].text == _("Alpha sort"):
			self["key_green"].text = _("Date sort")
			self.currentGrnText = _("Date sort")
			self["list"].setAlphaSort(True)
		else:
			self["key_green"].text = _("Alpha sort")
			self.currentGrnText = _("Alpha sort")
			self["list"].setAlphaSort(False)
		self.reloadList()

	def reloadList(self, mayOpenDialog=True):
		try:
			if os.path.exists(self.currentPathSel):
				self["list"].reload(self.currentPathSel + "/"*(self.currentPathSel != "/"))
				self.updateAfterKeyPress()
			else:
				if mayOpenDialog:
					self.session.open(MessageBox, _("Error: path %s not available!" % self.currentPathSel), MessageBox.TYPE_ERROR)
				else:
					Notifications.AddNotification(MessageBox, _("Error: path %s not available!" % self.currentPathSel), MessageBox.TYPE_ERROR)
		except Exception, e:
			spDebugOut("[spML] reloadList exception:\n" + str(e))

	def updateTitle(self):
		lotime = localtime()
		title = "[%02d:%02d] - " %(lotime[3],lotime[4])
		if os.path.exists(self.currentPathSel+"/"):
			stat = os.statvfs(self.currentPathSel+"/")
			free = stat.f_bavail * stat.f_bsize / 1048576	#1073741824
			if free >= 10240:	#unit in Giga bytes if more than 10 GB free
				title += "(%d GB) " %(free/1024)
			else:
				title += "(%d MB) " %(free)
			title += self.currentPathSel + (self.currentPathSel=="")*"/"

			if not self["list"].currentSelIsDirectory():
				try:
					service = self.getCurrent()
					size = os.path.getsize(service.getPath()) / 1024 / 1024
					title += " <%d MB>" % size
				except:
					pass

		self.setTitle(title)

	def moveMovie(self):
		current = self.getCurrent()
		if current is not None:
			selectedlist = self["list"].makeSelectionList()
			dialog = False
			if self["list"].currentSelIsDirectory():
				if current != selectedlist[0][0]:	# first selection != cursor pos?
					targetPath = self.currentPathSel
					if self["list"].getCurrentSelDir() == "..":
						targetPath = os.path.split(targetPath)[0]
					else:
						targetPath += "/" + self["list"].getCurrentSelDir()
					self.execFileOp(targetPath, selectedlist)
					self["list"].resetSelectionList()
				else:
					if len(selectedlist) == 1:
						self.session.open(MessageBox, _("How to move files:\nSelect some movies with the VIDEO-button, move the cursor on top of the destination directory and press yellow."), MessageBox.TYPE_ERROR, 10)
					else:
						dialog = True
			else:
				dialog = True
			if dialog:
				try:
					from Screens.LocationBox import LocationBox
					if len(selectedlist)==1 and self["list"].serviceBusy(selectedlist[0][0]): return
					self.tmpSelList = selectedlist
					self.session.openWithCallback(self.mvDirSelected, LocationBox, windowTitle= _("Select Location"), text = _("Choose directory"),
						filename = "", currDir = self.currentPathSel+"/", minFree = 0)
				except:
					self.session.open(MessageBox, _("How to move files:\nSelect some movies with the VIDEO-button, move the cursor on top of the destination directory and press yellow."), MessageBox.TYPE_ERROR, 10)

	def mvDirSelected(self, targetPath):
		if targetPath is not None:
			self.execFileOp(targetPath, self.tmpSelList)
			self["list"].resetSelectionList()
			self.tmpSelList = None

	def execFileOp(self, targetPath, selectedlist, op="move", purgeTrash=False):
		mvCmd = ""
		rmCmd = ""
		association = []
		for entry in selectedlist:
			service = entry[0]	# entry[1] = link path
			name = self["list"].getFileNameOfService(service).replace(".ts","")
			if name is not None:
				if op=="delete":	# target == trashcan
					if purgeTrash or self.currentPathSel == targetPath or mountpoint(self.currentPathSel) != mountpoint(targetPath):
						# direct delete from the trashcan or network mount (no copy to trashcan from different mountpoint)
						rmCmd += '; rm -f "'+ self.currentPathSel +"/"+ name +'."*'
					else:
						# create a time stamp with touch
						mvCmd += '; touch "'+ self.currentPathSel +"/"+ name +'."*'
						# move movie into the trashcan
						mvCmd += '; mv "'+ self.currentPathSel +"/"+ name +'."* "'+ targetPath +'/"'
					association.append((service, self.delCB))	# put in a callback for this particular movie
					self["list"].highlightService(True, "del", service)
					if config.suomipoeka.movie_hide_del.value:
						self["list"].removeService(service)
				elif op == "move":
					#if mountpoint(self.currentPathSel) == mountpoint(targetPath):	
					#	#self["list"].removeService(service)	# normal direct move
					#	pass
					#else:
					# different mountpoint? -> reset user&group
					if mountpoint(targetPath) != mountpoint(config.suomipoeka.movie_homepath.value):		# CIFS to HDD is ok!
						# need to change file ownership to match target filesystem file creation
						tfile = targetPath + "/owner_test"
						sfile = "\""+ self.currentPathSel +"/"+ name +".\"*"
						mvCmd += "; touch %s;ls -l %s | while read flags i owner group crap;do chown $owner:$group %s;done;rm %s" %(tfile,tfile,sfile,tfile)
					mvCmd += '; mv "'+ self.currentPathSel +"/"+ name +'."* "'+ targetPath +'/"'
					association.append((service, self.moveCB))	# put in a callback for this particular movie
					self["list"].highlightService(True, "move", service)
					if config.suomipoeka.movie_hide_mov.value:
						self["list"].removeService(service)
				self.lastPlayedCheck(service)
		if (mvCmd + rmCmd) != "":
			spTasker.shellExecute((mvCmd + rmCmd)[2:], association)	# first move, then delete if expiration limit is 0

	def moveCB(self, service):
		self["list"].highlightService(False, "move", service)	# remove the highlight
		if not config.suomipoeka.movie_hide_mov.value:
			self["list"].removeService(service)

	def delCB(self, service):
		self["list"].highlightService(False, "del", service)	# remove the highlight
		if not config.suomipoeka.movie_hide_del.value:
			self["list"].removeService(service)

	def emptyTrash(self, confirmed):
		try:
			if confirmed:
				# just make sure the user hasn't done the most stupid thing possible :)
				if not os.path.exists(config.suomipoeka.movie_trashpath.value + "/proc"):
					spTasker.shellExecute("rm -f \"%s/\"*" % config.suomipoeka.movie_trashpath.value)
				else:
					spDebugOut("[spML] emptyTrash WHOA! Trashcan path may be pointing to root, cancelling.")
		except Exception, e:
			spDebugOut("[spML] emptyTrash exception:\n" + str(e))

	def purgeExpiredFromTrash(self):
		try:
			if os.path.exists(config.suomipoeka.movie_trashpath.value):
				purgeCmd = ""
				dirlist = os.listdir(config.suomipoeka.movie_trashpath.value)
				for movie in dirlist:
					if movie[-3:] != ".ts": continue
					fullpath = config.suomipoeka.movie_trashpath.value +"/"+ movie
					currTime = localtime()
					expTime = localtime(os.stat(fullpath).st_mtime + 24*60*60*config.suomipoeka.movie_trashcan_limit.value)

					if currTime > expTime:
						purgeCmd += "; rm -f \"%s\"*" % fullpath.replace(".ts","")
				if purgeCmd != "":
					spTasker.shellExecute(purgeCmd[2:])
		except Exception, e:
			spDebugOut("[spML] purgeExpiredFromTrash exception:\n" + str(e))

	def trashcanCreate(self, confirmed):
		try:
			os.makedirs(config.suomipoeka.movie_trashpath.value)
			self.reloadList()	# reload to show the trashcan
		except Exception, e:
			self.session.open(MessageBox, _("Trashcan create failed. Check mounts and permissions."), MessageBox.TYPE_ERROR)
			spDebugOut("[spML] trashcanCreate exception:\n" + str(e))

